@@ -11,17 +11,21 @@
 /* DESCRIPTION
 /* .nf
 
+ /*
+  * Systemn library.
+  */
+#ifndef NO_STDINT_H
+#include <stdint.h>
+#endif
+
  /*
   * External interface.
   */
-#ifndef HASH_FNV_T
-#include <stdint.h>
-#ifdef USE_FNV_32BIT
+#ifdef NO_64_BITS
 #define HASH_FNV_T	uint32_t
-#else					/* USE_FNV_32BIT */
-#define HASH_FNV_T	uint64_t
-#endif					/* USE_FNV_32BIT */
-#endif					/* HASH_FNV_T */
+#else
+#define	HASH_FNV_T	uint64_t
+#endif
 
 extern HASH_FNV_T hash_fnv(const void *, size_t);
 
